package ca.mgamble.postman.api.message;

import java.util.ArrayList;
import java.util.List;

public class PostmanMessageBuilder {

    private List<String> to = new ArrayList<>();
    private List<String> cc = new ArrayList<>();
    private List<String> bcc = new ArrayList<>();
    private List<Attachment> attachments = new ArrayList<>();
    private List<EmbeddedImage> embeddedImages = new ArrayList<>();
    private List<Header> headers = new ArrayList<>();
    private String from;
    private String subject;
    private String tag;
    private String reply_to;
    private String plain_body;
    private String html_body;
    private boolean bounce;

    public PostmanMessageBuilder to(String to) {
        this.to.add(to);
        return this;
    }

    public PostmanMessageBuilder cc(String cc) {
        this.cc.add(cc);
        return this;
    }

    public PostmanMessageBuilder bcc(String bcc) {
        this.bcc.add(bcc);
        return this;
    }

    public PostmanMessageBuilder addAttachment(Attachment attachment) {
        this.attachments.add(attachment);
        return this;
    }

    public PostmanMessageBuilder addEmbeddedImage(EmbeddedImage embeddedImage) {
        this.embeddedImages.add(embeddedImage);
        return this;
    }

    public PostmanMessageBuilder addHeader(Header header) {
        this.headers.add(header);
        return this;
    }

    public PostmanMessageBuilder from(String from) {
        this.from = from;
        return this;
    }

    public PostmanMessageBuilder withSubject(String subject) {
        this.subject = subject;
        return this;
    }

    public PostmanMessageBuilder withTag(String tag) {
        this.tag = tag;
        return this;
    }

    public PostmanMessageBuilder withReplyTo(String reply_to) {
        this.reply_to = reply_to;
        return this;
    }

    public PostmanMessageBuilder withPlainBody(String plain_body) {
        this.plain_body = plain_body;
        return this;
    }

    public PostmanMessageBuilder withHtmlBody(String html_body) {
        this.html_body = html_body;
        return this;
    }

    public PostmanMessageBuilder withBounce(boolean bounce) {
        this.bounce = bounce;
        return this;
    }

    public PostmanMessage build() {
        return new PostmanMessage(to, cc, bcc, attachments, embeddedImages, headers, from, subject, tag, reply_to, plain_body, html_body, bounce);
    }
}
