package ca.mgamble.postman.api;

import ca.mgamble.postman.api.message.Attachment;
import ca.mgamble.postman.api.message.EmbeddedImage;
import ca.mgamble.postman.api.message.PostmanMessage;
import ca.mgamble.postman.api.message.PostmanMessageBuilder;
import ca.mgamble.postman.api.response.OperationStatus;
import ca.mgamble.postman.api.response.PostmanApiResponse;
import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Objects;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

class PostmanRawServiceTest {

    public static final String TO_ADRESS = "email@test.com";
    public static final String POSTMAN_URL = "https://postman.prmail.vn";
    public static final String POSTMAN_API_KEY = "API_KEY";
    public static final String FROM_ADRESS = "no-reply@my-domain.com";

    private PostmanService postmanService;

    @BeforeEach
    void setUp() {
        postmanService = new PostmanService(POSTMAN_URL, POSTMAN_API_KEY);
    }

    @Disabled("Only used to test sending email via API Postman, not to be run automatically")
    @Test
    public void sendMailTest_withEmbeddedImage_withAttachment_shouldSendRawMessage() throws Exception {
        // GIVEN
        byte[] googleLogo = IOUtils.toByteArray(Objects.requireNonNull(this.getClass().getClassLoader().getResourceAsStream("google.png")));
        byte[] csvFile = IOUtils.toByteArray(Objects.requireNonNull(this.getClass().getClassLoader().getResourceAsStream("test-csv.csv")));
        PostmanMessage message = new PostmanMessageBuilder()
                .from(FROM_ADRESS)
                .to(TO_ADRESS)
                .withSubject("Postman Java Google")
                .withPlainBody("Please view this email in a modern email client!")
                .withHtmlBody(getEmailBody())
                .addEmbeddedImage(new EmbeddedImage("google", "image/png", googleLogo))
                .addAttachment(new Attachment("test-csv.csv", "application/octet-stream", csvFile))
                .withTag("tag_name")
                .build();
        // WHEN
        PostmanApiResponse postmanApiResponse = postmanService.sendMessage(message);
        // THEN
        assertNotNull(postmanApiResponse);
        assertEquals(OperationStatus.SUCCESS, postmanApiResponse.getStatus());
    }

    public String getEmailBody() throws IOException {
        File file = new File(this.getClass().getClassLoader().getResource("email.html").getFile());
        return new String(Files.readAllBytes(file.toPath()));
    }
}
