package ca.mgamble.postman.api;

import ca.mgamble.postman.api.message.Attachment;
import ca.mgamble.postman.api.message.PostmanMessage;
import ca.mgamble.postman.api.message.PostmanMessageBuilder;
import ca.mgamble.postman.api.response.OperationStatus;
import ca.mgamble.postman.api.response.PostmanApiResponse;
import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.io.InputStream;
import java.util.Base64;
import java.util.Objects;

import static ca.mgamble.postman.api.PostmanRawServiceTest.FROM_ADRESS;
import static ca.mgamble.postman.api.PostmanRawServiceTest.POSTMAN_API_KEY;
import static ca.mgamble.postman.api.PostmanRawServiceTest.POSTMAN_URL;
import static ca.mgamble.postman.api.PostmanRawServiceTest.TO_ADRESS;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;

class PostmanServiceTest {

    private PostmanService postmanService;

    @BeforeEach
    void setUp() {
        postmanService = new PostmanService(POSTMAN_URL, POSTMAN_API_KEY);
    }

    @Disabled("Only used to test sending email via API Postman, not to be run automatically")
    @Test
    public void sendMailTest_withAttachments() throws Exception {
        // GIVEN
        PostmanMessage message = buildPostmanMessageWithAttachments(TO_ADRESS);
        // WHEN
        PostmanApiResponse postmanApiResponse = postmanService.sendMessage(message);
        // THEN
        assertNotNull(postmanApiResponse);
        assertEquals(OperationStatus.SUCCESS, postmanApiResponse.getStatus());
    }

    @Test
    public void onSendMail_withBadPostmanServiceUrl_shouldThrowException() throws Exception {
        // GIVEN
        PostmanMessage message = buildPostmanMessageWithAttachments("test@email.com");
        // WHEN
        Exception exception = assertThrows(Exception.class, () -> postmanService.sendMessage(message));
        // THEN
        assertEquals("Could not send message, server responded with 404", exception.getMessage());
    }

    private PostmanMessage buildPostmanMessageWithAttachments(String to) throws IOException {
        PostmanMessage message = new PostmanMessageBuilder()
                .from(FROM_ADRESS)
                .to(to)
                .withSubject("Testing with attachments")
                .withPlainBody("Please view this email in a modern email client!")
                .withHtmlBody("<html><h1>html body</h1><h2>Please check your attachments!</h2></html>")
                .withTag("tag_name")
                .build();

        try (InputStream inputStream = Objects.requireNonNull(this.getClass().getClassLoader().getResourceAsStream("test-csv.csv"))) {
            byte[] tblBordByte = IOUtils.toByteArray(inputStream);
            message.addAttachment(new Attachment("test-csv.csv", "application/octet-stream", new String(Base64.getEncoder().encode(tblBordByte))));
        }

        try (InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream("test-text.txt")) {
            byte[] tblBordByteTxt = IOUtils.toByteArray(Objects.requireNonNull(inputStream));
            message.addAttachment(new Attachment("test-text.txt", "text/plain", new String(Base64.getEncoder().encode(tblBordByteTxt))));
        }

        return message;
    }
}
